#!/usr/bin/python
# -*- coding: utf-8 -*-
"""Test Imports json schema validation."""
import pytest
from ve_utils.utype import UType as Ut
from jsonschema.exceptions import SchemaError
from vemonitor_m8.conf_manager.schema_validate import SchemaValidate
from vemonitor_m8.conf_manager.loader import Loader
from .schema_test_helper import SchemaTestHelper


@pytest.fixture(name="schema_manager", scope="class")
def schema_manager_fixture():
    """Json Schema test manager fixture"""
    class SchemaManager(SchemaTestHelper):
        """Json Schema test manager fixture Class"""

        def __init__(self):
            SchemaTestHelper.__init__(self)
            self.init_data()

        def init_data(self):
            """Init data"""
            self.schema = SchemaValidate.load_schema("imports")
            loader = Loader("test/conf/dummy_conf_dict.yaml")
            conf = loader.get_yaml_config()
            self.obj = conf.get('Imports')

        def get_string_import_values_helper(self, choice: str) -> list:
            """
            Return a list of string_import values
            to test jsonschema validation.

            The function returns different lists
            depending on the choice parameter,
            which is either good or bad.
            If choice is set to good,
            then strings that are expected to be valid will be returned.
            If choice is set to bad,
            then strings that are expected not to be valid will be returned
            :param self: Allow a function to refer to itself
            :param choice: Determine which list of values to return
            :return: A list of values that are bad or good string_import.
            """
            if choice == "good":
                return ["a.yaml", "a_b.yaml", "H1eL2lO.yaml", "0H1L_2lO.yaml"]
            else:
                return [
                    0, -1, -0.1, 0.1, 2.2,
                    "_hel lo", "_hello.yaml",
                    "hel lo.yaml", "hello.dev",
                    ".hello.yaml", ".hell.o.yaml",
                    False, None, dict(), tuple()
                ]

        def get_values_helper(self, key: str, choice: str) -> list:
            """
            Return a list of values to test json schema validation.

            Accepts two parameters, key and choice.
            - 'key' is a string that specifies
                the type of value to be returned.
            - 'choice' is also a string that specifies whether
                to return good or bad values for the specified key.

            :param self: Access variables that belongs to the class
            :param key: Determine which type of values to return
            :param choice: Determine which values to return
            :return: A list of values for the given key and choice
            :doc-author: Trelent
            """
            result = None
            if Ut.is_str(key) and choice in ['good', 'bad']:
                if key == "string_key":
                    result = self.get_string_key_values_helper(choice)
                elif key == "string_column":
                    result = self.get_string_columns_values_helper(choice)
                elif key == "string_text":
                    result = self.get_string_text_values_helper(choice)
                elif key == "string_import":
                    result = self.get_string_import_values_helper(choice)
                elif key == "positive_number":
                    result = self.get_positive_number_values_helper(choice)
                elif key == "positive_integer":
                    result = self.get_positive_integer_values_helper(choice)
            return result

    return SchemaManager()


class TestImportsSchema:
    """Test Imports json schema validation."""

    def test_bad_file_key(self):
        """Test bad file key"""
        with pytest.raises(SchemaError):
            SchemaValidate.load_schema("bad_key")

    def test_data_validation(self, schema_manager):
        """Test data validation"""
        assert Ut.is_list(
            SchemaValidate.validate_data(schema_manager.obj, "imports"),
            not_null=True
        )

    def test_string_import_pattern(self, schema_manager):
        """Test string_import values to validate patterns"""
        datas = [
            (0, schema_manager.obj)
        ]
        schema_manager.run_test_values(datas=datas, key="string_import")



"""Vemonitor_m8 Setup"""
import pathlib
from setuptools import setup

# The directory containing this file
HERE = pathlib.Path(__file__).parent

# The text of the README file
README = (HERE / "README.md").read_text()

setup(
    name='vemonitor_m8',
    version="0.1.2",
    description='Solar Plant Monitoring',
    url='https://github.com/vemonitor/vemonitor_m8',
    author='Eli Serra',
    author_email='eli.serra173@gmail.com',
    long_description=README,
    long_description_content_type="text/markdown",
    classifiers=[
        "License :: OSI Approved :: Apache Software License",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
    ],
    license='Apache',
    py_modules=[
        'vemonitor_m8.conf_manager.loader',
        'vemonitor_m8.conf_manager.schema_validate',
        'vemonitor_m8.conf_manager.schema_validate_selector',
        'vemonitor_m8.core.exceptions',
        'vemonitor_m8.core.utils',
        'vemonitor_m8.models.workers',
        'vemonitor_m8.models.item_dict'
    ],
    packages=[
        'test',
        'vemonitor_m8'
    ],
    package_dir={
        'vemonitor_m8': 'vemonitor_m8',
        'test': 'test'
    },
    package_data={
        'vemonitor_m8': [
            'conf_manager/confFiles/*.yaml',
            'conf_manager/confFiles/*.yml'
        ],
        'test': [
            'conf/*.yaml',
            'conf/*.yml'
        ]
    },
    include_package_data=True,
    install_requires=[
        'jsonschema>=4.23.0',
        'pyyaml>=6.0.2',
        'simplejson>=3.19.2',
        'redis>=5.0.8',
        'vedirect_m8>=1.3.4',
        've-utils>=2.5.3'
    ],
    extras_require={
        "TEST": [
            "pytest>=8.3.2",
            "pytest-cov>=5.0.0",
            "coverage>=7.6.1",
            "setuptools>=72.1.0",
            "twine>=5.1.1",
            "wheel>=0.44.0",
            "flake8>=7.1.1"
        ]
    },
    python_requires='>3.5.2',
    zip_safe=False
)



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""Helper class to test json schemas validation."""
from typing import Optional, Union
import pytest
from ve_utils.utype import UType as Ut
from jsonschema.exceptions import ValidationError
from vemonitor_m8.conf_manager.schema_validate import SchemaValidate


class SchemaTestHelper:
    """Helper class to test json schemas validation."""

    def __init__(self):
        self.schema = None
        self.obj = None

    def get_values_helper(self, key: str, choice: str) -> Optional[list]:
        """
        Return a list of values to test json schema validation.

        Accepts two parameters, key and choice.
         - 'key' is a string that specifies the type of value to be returned.
         - 'choice' is also a string that specifies whether
            to return good or bad values for the specified key.

        :param self: Access variables that belongs to the class
        :param key: Determine which type of values to return
        :param choice: Determine which values to return
        :return: A list of values for the given key and choice
        :doc-author: Trelent
        """
        result = None
        if Ut.is_str(key) and choice in ['good', 'bad']:
            if key == "string_key":
                result = self.get_string_key_values_helper(choice)
            elif key == "string_column":
                result = self.get_string_columns_values_helper(choice)
            elif key == "string_text":
                result = self.get_string_text_values_helper(choice)
            elif key == "positive_number":
                result = self.get_positive_number_values_helper(choice)
            elif key == "positive_integer":
                result = self.get_positive_integer_values_helper(choice)
        return result

    def get_string_text_values_helper(self, choice: str) -> list:
        """
        Return a list of string_text values to test json schema validation.

        The function returns different lists depending on the choice parameter,
        which is either good or bad.
        If choice is set to good,
        then strings that are expected to be valid will be returned.
        If choice is set to bad,
        then strings that are expected not to be valid will be returned.

        :param self: Allow a function to refer to itself
        :param choice: Determine which list of values to return
        :return: A list of values that are bad or good string_text
        """
        #
        if choice == "good":
            return [
                "hello", "HeLlO", "H1eL2lO", "0H1eL2lO", "0H1e_L2lO",
                "H1_eL 2-l/O", "0H1e-L2lO", "0H1e_L2lO", "0H1e L2lO"
            ]
        else:
            return [
                1, 0, None, "_hello", "hello_", "a&",
                "c^", "m%", "Ad8M.s+dsd", "Ad8M$ sdsd",
                "[a-z]", "%shello", "hello%sworld"
            ]

    def get_string_key_values_helper(self, choice: str) -> list:
        """
        Return a list of string_key values to test json schema validation.

        The function returns different lists depending on the choice parameter,
        which is either good or bad.
        If choice is set to good,
        then strings that are expected to be valid will be returned.
        If choice is set to bad,
        then strings that are expected not to be valid will be returned.

        :param self: Allow a function to refer to itself
        :param choice: Determine which list of values to return
        :return: A list of values that are bad or good string_key
        """
        if choice == "good":
            return [
                "hello", "HeLlO", "H1eL2lO", "0H1eL2lO", "0H1e_L2lO"
            ]
        else:
            return [
                1, 0, None, "_hello", "hello_", "hello#", "a&",
                "c$", "m%", "Ad8M.sdsd", "Ad8M sdsd", "[a-z]",
                "%shello"
            ]

    def get_string_columns_values_helper(self, choice: str) -> list:
        """
        Return a list of string_columns values to test json schema validation.

        The function returns different lists depending on the choice parameter,
        which is either good or bad.
        If choice is set to good,
        then strings that are expected to be valid will be returned.
        If choice is set to bad,
        then strings that are expected not to be valid will be returned.

        :param self: Allow a function to refer to itself
        :param choice: Determine which list of values to return
        :return: A list of values that are bad or good string_columns
        """
        if choice == "good":
            return [
                "hello", "HeLlO", "H1eL2lO", "0H1eL2lO", "0H1e_L2lO",
                "#H1eL2lO", "0H1e#L2lO", "0H1e_L2lO#",
            ]
        else:
            return [
                1, 0, None, "_hello", "hello_", "a&",
                "c^", "m%", "Ad8M.sdsd", "Ad8M sdsd",
                "[a-z]", "%shello", "hello%sworld"
            ]

    def get_positive_number_values_helper(self, choice: str) -> list:
        """
        Return a list of positive_number values to test jsonschema validation.

        The function returns different lists depending on the choice parameter,
        which is either good or bad.
        If choice is set to good,
        then strings that are expected to be valid will be returned.
        If choice is set to bad,
        then strings that are expected not to be valid will be returned.

        :param self: Allow a function to refer to itself
        :param choice: Determine which list of values to return
        :return: A list of values that are bad or good positive_number
        """
        if choice == "good":
            return [1, 3, 10, 1.2]
        else:
            return [
                0, -1, -0.1, "_hel lo",
                False, None, dict(), tuple()
            ]

    def get_positive_integer_values_helper(self, choice: str) -> list:
        """
        Return a list of positive_integer values to test jsonschema validation.

        The function returns different lists depending on the choice parameter,
        which is either good or bad.
        If choice is set to good,
        then strings that are expected to be valid will be returned.
        If choice is set to bad,
        then strings that are expected not to be valid will be returned.

        :param self: Allow a function to refer to itself
        :param choice: Determine which list of values to return
        :return: A list of values that are bad or good positive_integer
        """
        result = None
        if choice == "good":
            result = [1, 3, 10]
        else:
            result = [
                0, -1, -0.1, 0.1, 2.2, "_hel lo",
                False, None, dict(), tuple()
            ]
        return result

    def _run_test_bad_value_data(self,
                                 key: Union[str, int],
                                 data: any,
                                 new_val: any
                                 ) -> None:
        val = data[key]
        data[key] = new_val
        with pytest.raises(ValidationError):
            SchemaValidate.validate_data_from_schema(self.obj, self.schema)
        data[key] = val
        return data[key] == val

    def _run_test_good_value_data(self,
                                  key: Union[str, int],
                                  data: any,
                                  new_val: any
                                  ) -> None:
        val = data[key]
        data[key] = new_val
        SchemaValidate.validate_data_from_schema(self.obj, self.schema)
        data[key] = val
        return data[key] == val

    def _run_assert_values_data(self,
                                key: Union[str, int],
                                item: Union[dict, list],
                                list_values: list,
                                choice: str,
                                restrict: str
                                ):
        if Ut.is_str(key) or Ut.is_int(key) \
                and (Ut.is_dict(item, not_null=True)
                     or Ut.is_list(item, not_null=True))\
                and Ut.is_list(list_values, not_null=True):

            for val_tst in list_values:
                if choice == "good" and restrict != "bad":
                    assert self._run_test_good_value_data(key, item, val_tst)
                elif choice == "bad" and restrict != "good":
                    assert self._run_test_bad_value_data(key, item, val_tst)
                else:
                    if not ((choice == "good" and restrict == "bad")
                            or (choice == "bad" and restrict == "good")):
                        raise ValueError(
                            f"Fatal error: Unable to evaluate {choice} value "
                            f"on key {key}. With restriction value {restrict}"
                        )
        else:
            raise ValueError(
                "Fatal error: Unable to evaluate good values "
                f"on key {key}"
            )

    def run_test_values_data(self,
                             datas: list,
                             list_values: list,
                             choice: str
                             ):
        """Apply and test values."""
        if Ut.is_list(datas, not_null=True)\
                and Ut.is_list(list_values, not_null=True):
            for data_item in datas:
                key, item, restrict = None, None, None
                if Ut.is_tuple(data_item):
                    if len(data_item) == 2:
                        key, item = data_item
                    elif len(data_item) == 3:
                        key, item, restrict = data_item
                    else:
                        raise ValueError(
                            "Fatal error: Unable to evaluate good values "
                            f"on key {key}, "
                            "items tuple must have a length of 2 or 3. "
                            "( key: str or int, data: list or dict )"
                        )
                    self._run_assert_values_data(
                        key,
                        item,
                        list_values,
                        choice,
                        restrict
                    )
                else:
                    raise ValueError(
                        "Fatal error: Unable to evaluate good values "
                        f"on key {key}, items must be a tuple. "
                        "( key: str or int, data: list or dict )"
                    )
        else:
            raise ValueError(
                "Fatal error: Unable to evaluate good values "
                f"on list values {list_values}"
            )

    def run_test_values(self, datas: list, key: str):
        """Apply and test all values."""
        self.run_test_values_data(
            datas=datas,
            list_values=self.get_values_helper(key, 'good'),
            choice='good'
        )

        self.run_test_values_data(
            datas=datas,
            list_values=self.get_values_helper(key, 'bad'),
            choice='bad'
        )



"""Test active_connectors helper module."""
import pytest
from ve_utils.utype import UType as Ut
from vemonitor_m8.workers.active_connectors import ActiveConnectors


@pytest.fixture(name="helper_manager", scope="class")
def helper_manager_fixture():
    """Json Schema test manager fixture"""
    class HelperManager:
        """Json Helper test manager fixture Class"""

        def __init__(self):
            self.obj = ActiveConnectors()

    return HelperManager()


class TestActiveConnectors:
    """Test ActiveConnectors model class."""

    def test_has_items(self, helper_manager):
        """Test has_items method """
        helper_manager.obj.add_item(
          key=("worker_name", "worker_key"),
          value=("worker_status", "worker_interval")
        )
        assert helper_manager.obj.has_items() is True

    def test_has_item_key(self, helper_manager):
        """Test has_item_key method """
        helper_manager.obj.add_item(
          key=("worker_name", "worker_key"),
          value=("worker_status", "worker_interval")
        )
        assert helper_manager.obj.has_item_key(
            key=("worker_name", "worker_key")
        ) is True

    def test_init_item(self, helper_manager):
        """Test init_item method """
        helper_manager.obj.add_item(
          key=("worker_name", "worker_key"),
          value=("worker_status", "worker_interval")
        )
        helper_manager.obj.init_item(reset=True)
        assert helper_manager.obj.has_items() is False

    def test_add_item(self, helper_manager):
        """Test add_item method """
        result = helper_manager.obj.add_item(
          key=("worker_name", "worker_key"),
          value=("worker_status", "worker_interval")
        )
        assert result is True
        assert helper_manager.obj.has_items() is True
        assert helper_manager.obj.has_item_key(
            key=("worker_name", "worker_key")
        ) is True

        result = helper_manager.obj.add_item(
          key="worker_name",
          value="worker_name"
        )
        assert result is False

    def test_get_item(self, helper_manager):
        """Test get_item method """
        result = helper_manager.obj.add_item(
          key=("worker_name", "worker_key"),
          value=("worker_status", "worker_interval")
        )
        assert result is True
        result = helper_manager.obj.get_item(
          key=("worker_name", "worker_key")
        )

        assert result == ("worker_status", "worker_interval")

    def test_loop_on_items(self, helper_manager):
        """Test loop_on_items method """
        result = helper_manager.obj.add_item(
          key=("worker_name", "worker_key"),
          value=("worker_status", "worker_interval")
        )
        assert result is True
        for key, item in helper_manager.obj.loop_on_items():
            assert key == ("worker_name", "worker_key")
            assert item == ("worker_status", "worker_interval")

    def test_get_items(self, helper_manager):
        """Test get_items method """
        key = ("worker_name", "worker_key")
        item = ("worker_status", "worker_interval")
        result = helper_manager.obj.add_item(
          key=key,
          value=item
        )
        assert result is True
        result = helper_manager.obj.get_items()
        assert Ut.is_dict(result, eq=1)
        assert result.get(key) == item

    def test_is_valid_item_key(self, helper_manager):
        """Test is_valid_item_key method """
        assert helper_manager.obj.is_valid_item_key(
            key=("a", "b")
        ) is True

        assert helper_manager.obj.is_valid_item_key(
            key=("", "")
        ) is False

        assert helper_manager.obj.is_valid_item_key(
            key="a"
        ) is False

        assert helper_manager.obj.is_valid_item_key(
            key=None
        ) is False

    def test_is_valid_item_value(self, helper_manager):
        """Test is_valid_item_value method """
        assert helper_manager.obj.is_valid_item_value(
            value=("a", "b")
        ) is True

        assert helper_manager.obj.is_valid_item_value(
            value=("", "")
        ) is False

        assert helper_manager.obj.is_valid_item_value(
            value="a"
        ) is False

        assert helper_manager.obj.is_valid_item_value(
            value=None
        ) is False



"""Test config model module."""
import inspect
from os import path as Opath
import pytest
from ve_utils.utype import UType as Ut
from vemonitor_m8.core.exceptions import SettingInvalidException
from vemonitor_m8.models.config import Config
from vemonitor_m8.conf_manager.config_loader import ConfigLoader


@pytest.fixture(name="helper_manager", scope="class")
def helper_manager_fixture():
    """Json Schema test manager fixture"""
    class HelperManager:
        """Json Helper test manager fixture Class"""

        def __init__(self):
            self.obj = Config()
            self.test_path = None
            self.loader = None

        def init_loader(self):
            """Init config loader module"""
            current_script_path = Opath.dirname(
                Opath.abspath(
                    inspect.getfile(inspect.currentframe())
                )
            )
            self.test_path = Opath.join(
                current_script_path,
                "conf"
            )
            self.loader = ConfigLoader(self.test_path)

        def init_config_with_loader(self):
            """Init config loader module"""
            self.init_loader()
            settings = self.loader.get_settings_from_schema(
                app_name="batSerialMonitor"
            )
            self.obj = settings

    return HelperManager()


class TestConfigAppBlocks:
    """Test ConfigAppBlocks model class."""

    def test_set_app_blocks(self, helper_manager):
        """Test set_app_blocks method """
        helper_manager.init_config_with_loader()
        result = helper_manager.obj.set_app_blocks(
            app_blocks=[]
        )
        assert result is False

    def test_get_app_block_by_name(self, helper_manager):
        """Test get_app_block_by_name method """
        helper_manager.init_config_with_loader()
        data = helper_manager.obj.get_app_block_by_name(
            block_name="bmvFakeSerial"
        )
        assert Ut.is_dict(data, not_null=True)
        assert data.get('name') == "bmvFakeSerial"
        assert data.get('app') == 'batSerialMonitor'

    def test_get_app_block_by_app(self, helper_manager):
        """Test get_app_block_by_app method """
        helper_manager.init_config_with_loader()
        data = helper_manager.obj.get_app_block_by_app(
            app_name="batSerialMonitor"
        )
        assert Ut.is_dict(data, not_null=True)
        assert data.get('name') == "bmvFakeSerial"
        assert data.get('app') == 'batSerialMonitor'

    def test_get_app_blocks_columns(self, helper_manager):
        """Test get_app_blocks_columns method """
        helper_manager.init_config_with_loader()
        data = helper_manager.obj.get_app_blocks_columns()
        assert Ut.is_list(data, eq=23)

    def test_get_app_blocks_sources(self, helper_manager):
        """Test get_app_blocks_columns method """
        helper_manager.init_config_with_loader()
        data = helper_manager.obj.get_app_blocks_sources()
        assert Ut.is_dict(data, eq=2)
        assert data.get('serial') == ['bmv700']
        assert data.get('redis') == ['local']


class TestConfigAppConnectors:
    """Test ConfigAppConnectors model class."""
    def test_set_app_connectors(self, helper_manager):
        """Test set_app_connectors method """
        helper_manager.init_config_with_loader()
        result = helper_manager.obj.set_app_connectors(
            app_connectors={}
        )
        assert result is False

    def test_has_app_connector(self, helper_manager):
        """Test has_app_connector method """
        helper_manager.init_config_with_loader()

        assert helper_manager.obj.has_app_connectors() is True

        assert helper_manager.obj.has_app_connector_key(
            key="redis"
        ) is True

        assert helper_manager.obj.has_app_connector_key_item(
            key="redis",
            item="local"
        ) is True

    def test_get_app_connector_by_key(self, helper_manager):
        """Test get_app_connector_by_key method """
        helper_manager.init_config_with_loader()
        result = helper_manager.obj.get_app_connector_by_key(
            key="redis"
        )
        assert Ut.is_dict(result, not_null=True)
        assert Ut.is_dict(result.get('local'), not_null=True)
        assert result['local'].get('host') == '127.0.0.1'
        assert result['local'].get('port') == 6379

    def test_get_app_connector_by_key_item(self, helper_manager):
        """Test get_app_connector_by_key_item method """
        helper_manager.init_config_with_loader()
        result = helper_manager.obj.get_app_connector_by_key_item(
            key="redis",
            item="local"
        )
        assert Ut.is_dict(result, not_null=True)
        assert result.get('host') == '127.0.0.1'
        assert result.get('port') == 6379

        result = helper_manager.obj.get_app_connector_by_key_item(
            key="bad_key",
            item="bad_item"
        )
        assert result is None

    def test_get_app_connector_by_sources(self, helper_manager):
        """Test get_app_connector_by_sources method """
        helper_manager.init_config_with_loader()
        sources = helper_manager.obj.get_app_blocks_sources()
        result = helper_manager.obj.get_app_connector_by_sources(
            sources=sources
        )
        sources_keys = list(sources.keys())
        connectors_keys = list(result.keys())
        assert Ut.is_dict(result, not_null=True)
        assert sources_keys == connectors_keys
        assert Ut.is_dict(result['serial'].get('bmv700'), not_null=True)
        assert Ut.is_dict(result['redis'].get('local'), not_null=True)

    def test_reduce_app_connector_from_sources(self, helper_manager):
        """Test reduce_app_connector_from_sources method """
        helper_manager.init_config_with_loader()
        data = helper_manager.obj.reduce_app_connector_from_sources(
            app_connector=helper_manager.obj.app_connectors
        )
        assert Ut.is_dict(data, eq=2)
        assert data == helper_manager.obj.app_connectors

    def test_set_and_reduce_app_connectors(self, helper_manager):
        """Test set_and_reduce_app_connectors method """
        helper_manager.init_config_with_loader()
        result = helper_manager.obj.set_and_reduce_app_connectors(
            app_connector=helper_manager.obj.app_connectors
        )
        assert result is True

        result = helper_manager.obj.set_and_reduce_app_connectors(
            app_connector={}
        )
        assert result is False


class TestConfigDataStructures:
    """Test ConfigDataStructures model class."""
    def test_set_data_structures(self, helper_manager):
        """Test set_data_structures method """
        helper_manager.init_config_with_loader()
        helper_manager.obj.set_data_structures(
            data_structures=helper_manager.obj.data_structures
        )

        with pytest.raises(SettingInvalidException):
            helper_manager.obj.set_data_structures(
                data_structures={'a': 2}
            )

    def test_has_data_structures(self, helper_manager):
        """Test has_data_structures method """
        helper_manager.init_config_with_loader()
        assert helper_manager.obj.has_data_structures() is True

    def test_has_data_structures_point_key(self, helper_manager):
        """Test has_data_structures_point_key method """
        helper_manager.init_config_with_loader()
        assert helper_manager.obj.has_data_structures_point_key(
            key='V'
        ) is True

    def test_get_data_structures_point_by_columns(self, helper_manager):
        """Test get_data_structures_point_by_columns method """
        helper_manager.init_config_with_loader()
        columns = ['V', 'I', 'P']
        result = helper_manager.obj.get_data_structures_point_by_columns(
            columns=columns
        )
        assert Ut.is_dict(result, eq=3)
        assert Ut.is_dict(result.get('V'), not_null=True)
        assert Ut.is_dict(result.get('I'), not_null=True)
        assert Ut.is_dict(result.get('P'), not_null=True)

    def test_is_missing_data_structure(self, helper_manager):
        """Test is_missing_data_structure method """
        helper_manager.init_config_with_loader()
        columns = ['V', 'I', 'P']
        result = helper_manager.obj.is_missing_data_structure(
            data_structures=helper_manager.obj.data_structures,
            cols=columns
        )
        assert result is False

        with pytest.raises(SettingInvalidException):
            helper_manager.obj.is_missing_data_structure(
                data_structures=helper_manager.obj.data_structures,
                cols=['V', 'bad_point']
            )


class TestConfigConfigBatteryBanks:
    """Test ConfigBatteryBanks model class."""
    def test_has_battery_banks(self, helper_manager):
        """Test has_battery_banks method """
        helper_manager.init_config_with_loader()
        assert helper_manager.obj.has_battery_banks() is True

    def test_validate_battery_banks(self, helper_manager):
        """Test validate_battery_banks method """
        helper_manager.init_config_with_loader()

        with pytest.raises(SettingInvalidException):
            helper_manager.obj.validate_battery_banks(
                battery_banks={}
            )

        with pytest.raises(SettingInvalidException):
            helper_manager.obj.validate_battery_banks(
                battery_banks={'a': 1}
            )

    def test_set_battery_banks(self, helper_manager):
        """Test set_battery_banks method """
        helper_manager.init_config_with_loader()

        with pytest.raises(SettingInvalidException):
            helper_manager.obj.set_battery_banks(
                battery_banks={}
            )

    def test_get_battery_banks_from_args(self, helper_manager):
        """Test get_battery_banks_from_args method """
        helper_manager.init_config_with_loader()

        battery_banks = helper_manager.obj.get_battery_banks_from_args(
            args=helper_manager.obj.app_blocks[0].get('args')
        )
        assert Ut.is_dict(battery_banks, not_null=True)
        assert battery_banks.get('battery_type') == 'flooded'
        assert battery_banks.get('battery_key') == 'project1'


class TestConfigSolarPlants:
    """Test ConfigSolarPlants model class."""



"""Test ConfigLoaderHelper class."""
import inspect
from os import path as Opath
import pytest
from ve_utils.utype import UType as Ut
from jsonschema.exceptions import ValidationError
from vemonitor_m8.conf_manager.config_loader_helper\
    import ConfigLoaderHelper as clh
from vemonitor_m8.conf_manager.config_loader import ConfigLoader
from vemonitor_m8.core.exceptions import SettingInvalidException


@pytest.fixture(name="helper_manager", scope="class")
def helper_manager_fixture():
    """Json Schema test manager fixture"""
    class HelperManager:
        """Json Helper test manager fixture Class"""
        def __init__(self):
            current_script_path = Opath.dirname(
                Opath.abspath(
                    inspect.getfile(inspect.currentframe())
                )
            )
            self.test_path = Opath.join(
                current_script_path,
                "conf"
            )

    return HelperManager()


class TestConfigLoaderHelper:
    """Test ConfigLoaderHelper class."""

    def test_get_app_blocks_by_app_or_name(self, helper_manager):
        """Test get_app_blocks_by_app_or_name method """
        c_loader = ConfigLoader(helper_manager.test_path)
        settings = c_loader.get_settings_from_schema(
            app_name="batSerialMonitor"
        )
        res = clh.get_app_blocks_by_app_or_name(app_blocks=settings.app_blocks)
        assert res == settings.app_blocks

        res = clh.get_app_blocks_by_app_or_name(
            app_blocks=settings.app_blocks,
            block_name='bmvFakeSerial'
        )
        assert res == settings.app_blocks

        res = clh.get_app_blocks_by_app_or_name(
            app_blocks=settings.app_blocks,
            app_name='batSerialMonitor'
        )
        assert res == settings.app_blocks

        res = clh.get_app_blocks_by_app_or_name(
            app_blocks=settings.app_blocks,
            app_name='bad_app_name'
        )
        assert Ut.is_list(res) and len(res) == 0

        res = clh.get_app_blocks_by_app_or_name(
            app_blocks=settings.app_blocks,
            block_name='bad_block_name'
        )
        assert Ut.is_list(res) and len(res) == 0

        res = clh.get_app_blocks_by_app_or_name(app_blocks=list())
        assert res is None

    def test_get_app_blocks_by_app_or_name_errors(self):
        """Test get_app_blocks_by_app_or_name method """
        with pytest.raises(ValidationError):
            clh.get_app_blocks_by_app_or_name(app_blocks=['1', '2'])

    def test_get_args_objects_from_conf(self, helper_manager):
        """
        Test get_args_objects_from_conf method
        """
        c_loader = ConfigLoader(helper_manager.test_path)
        settings = c_loader.get_settings_from_schema(
            app_name="batSerialMonitor"
        )
        imp_conf = c_loader._get_settings_from_file()
        res = clh.get_args_objects_from_conf(
            args=settings.app_blocks[0].get('args'),
            conf=imp_conf
        )
        assert Ut.is_dict(res, not_null=True)

        res = clh.get_args_objects_from_conf(
            args=dict(),
            conf=imp_conf
        )
        assert res is None

        res = clh.get_args_objects_from_conf(
            args=settings.app_blocks[0].get('args'),
            conf=dict()
        )
        assert res is None

    def test_get_app_blocks_args_objects(self, helper_manager):
        """Test get_app_blocks_args_objects method."""
        c_loader = ConfigLoader(helper_manager.test_path)
        settings = c_loader.get_settings_from_schema(
            app_name="batSerialMonitor"
        )
        imp_conf = c_loader._get_settings_from_file()
        res = clh.get_app_blocks_args_objects(
            app_blocks=settings.app_blocks,
            conf=imp_conf
        )
        assert Ut.is_dict(res, not_null=True)

        res = clh.get_app_blocks_args_objects(
            app_blocks=settings.app_blocks,
            conf=dict()
        )
        assert res is None

        with pytest.raises(ValidationError):
            clh.get_app_blocks_args_objects(
                app_blocks=['1', '2'],
                conf=imp_conf
            )

    def test_get_app_conector_from_source(self, helper_manager):
        """Test get_app_conector_from_source method"""
        c_loader = ConfigLoader(helper_manager.test_path)
        settings = c_loader.get_settings_from_schema(
            app_name="batSerialMonitor"
        )
        imp_conf = c_loader._get_settings_from_file()
        sources = settings.get_app_blocks_sources()
        res = clh.get_app_conector_from_source(
            conector=imp_conf.get('appConnectors').get('serial'),
            sources=sources.get('serial')
        )
        assert Ut.is_dict(res, not_null=True)

        with pytest.raises(SettingInvalidException):
            clh.get_app_conector_from_source(
                conector=imp_conf.get('appConnectors'),
                sources=sources.get('serial')
            )

        with pytest.raises(SettingInvalidException):
            clh.get_app_conector_from_source(
                conector=dict(),
                sources=sources.get('serial')
            )

        with pytest.raises(SettingInvalidException):
            clh.get_app_conector_from_source(
                conector=imp_conf.get('appConnectors'),
                sources=list()
            )

    def test_get_app_connectors_from_sources(self, helper_manager):
        """Test get_app_connectors_from_sources method"""
        c_loader = ConfigLoader(helper_manager.test_path)
        settings = c_loader.get_settings_from_schema(
            app_name="batSerialMonitor"
        )
        imp_conf = c_loader._get_settings_from_file()
        res = clh.get_app_connectors_from_sources(
            app_conectors=imp_conf.get('appConnectors'),
            sources=settings.get_app_blocks_sources()
        )
        assert Ut.is_dict(res, not_null=True)

    def test_is_battery_banks(self, helper_manager):
        """Test is_battery_banks method"""
        c_loader = ConfigLoader(helper_manager.test_path)
        imp_conf = c_loader._get_settings_from_file()
        assert clh.is_battery_banks(imp_conf.get('batteryBanks')) is True

        assert clh.is_battery_banks(dict()) is False

        assert clh.is_battery_banks({'a': 1}) is False

    def test_is_battery_banks_items(self, helper_manager):
        """Test is_battery_banks_items method"""
        c_loader = ConfigLoader(helper_manager.test_path)
        imp_conf = c_loader._get_settings_from_file()
        assert clh.is_battery_banks_items(imp_conf.get('batteryBanks')) is True

    def test_is_battery_banks_items_key(self, helper_manager):
        """Test is_battery_banks_items_key method"""
        c_loader = ConfigLoader(helper_manager.test_path)
        imp_conf = c_loader._get_settings_from_file()
        assert clh.is_battery_banks_items_key(
            key='project1',
            battery_bank=imp_conf.get('batteryBanks')) is True

    def test_get_battery_banks_items_key(self, helper_manager):
        """Test get_battery_banks_items_key method"""
        c_loader = ConfigLoader(helper_manager.test_path)
        imp_conf = c_loader._get_settings_from_file()
        res = clh.get_battery_banks_items_key(
            key='project1',
            battery_bank=imp_conf.get('batteryBanks'))
        assert Ut.is_dict(res, not_null=True)

    def test_is_battery_items(self, helper_manager):
        """Test is_battery_items method"""
        c_loader = ConfigLoader(helper_manager.test_path)
        imp_conf = c_loader._get_settings_from_file()
        assert clh.is_battery_items(imp_conf.get('batteryBanks')) is True

    def test_is_battery_items_key(self, helper_manager):
        """Test is_battery_items_key method"""
        c_loader = ConfigLoader(helper_manager.test_path)
        imp_conf = c_loader._get_settings_from_file()
        assert clh.is_battery_items_key(
            key='project1',
            battery_bank=imp_conf.get('batteryBanks')
        ) is True

    def test_get_battery_items_key(self, helper_manager):
        """Test get_battery_items_key method"""
        c_loader = ConfigLoader(helper_manager.test_path)
        imp_conf = c_loader._get_settings_from_file()
        res = clh.get_battery_items_key(
            key='project1',
            battery_bank=imp_conf.get('batteryBanks')
        )
        assert Ut.is_dict(res, not_null=True)

    def test_set_battery_item(self, helper_manager):
        """Test set_battery_item method"""
        c_loader = ConfigLoader(helper_manager.test_path)
        imp_conf = c_loader._get_settings_from_file()
        res = clh.set_battery_item(
            key='project1',
            battery=dict(),
            battery_bank=imp_conf.get('batteryBanks')
        )
        assert Ut.is_dict(res, not_null=True)

    def test_get_battery_bank_from_arg(self, helper_manager):
        """Test get_battery_bank_from_arg method"""
        c_loader = ConfigLoader(helper_manager.test_path)
        imp_conf = c_loader._get_settings_from_file()
        res = clh.get_battery_bank_from_arg(
            arg='project1',
            battery_bank=imp_conf.get('batteryBanks')
        )
        assert Ut.is_dict(res, not_null=True)

        with pytest.raises(ValidationError):
            clh.get_battery_bank_from_arg(
                arg='project1',
                battery_bank={'a': 1}
            )

        with pytest.raises(SettingInvalidException):
            clh.get_battery_bank_from_arg(
                arg='bad_arg',
                battery_bank=imp_conf.get('batteryBanks')
            )

    def test__is_filtered_key(self):
        """Test _is_filtered_key method"""
        res = clh._is_filtered_key(
            key='project1',
            check_keys=None
        )
        assert res is True

        res = clh._is_filtered_key(
            key='project1',
            check_keys=['project1', 'project2', 'project2']
        )
        assert res is True

        res = clh._is_filtered_key(
            key='project5',
            check_keys=['project1', 'project2', 'project2']
        )
        assert res is False

        res = clh._is_filtered_key(
            key='',
            check_keys=['project1', 'project2', 'project2']
        )
        assert res is False

    def test__check_column_keys(self):
        """Test _check_column_keys method"""
        res = clh._check_column_keys(
            data=dict()

        )
        assert Ut.is_dict(res)

        with pytest.raises(SettingInvalidException):
            clh._check_column_keys(
                data=None
            )

        with pytest.raises(SettingInvalidException):
            clh._check_column_keys(
                data={"key": 1}
            )

    def test__check_column_points(self):
        """Test _check_column_points method"""
        res = clh._check_column_points(
            data={"key": 1}
        )
        assert Ut.is_dict(res)

        with pytest.raises(SettingInvalidException):
            clh._check_column_points(
                data=None
            )

    def test_get_data_structure(self, helper_manager):
        """Test get_data_structure method"""
        c_loader = ConfigLoader(helper_manager.test_path)
        settings = c_loader.get_settings_from_schema(
            app_name="batSerialMonitor"
        )
        res = clh.get_data_structure(
            data_structure=settings.data_structures
        )
        assert Ut.is_dict(res, not_null=True)

        with pytest.raises(ValidationError):
            clh.get_data_structure(
                data_structure=None
            )



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""Test Loader class."""
import inspect
from os import path as Opath
import pytest
from ve_utils.utype import UType as Ut
from vemonitor_m8.conf_manager.loader import Loader
from vemonitor_m8.core.exceptions import YAMLFileNotFound


@pytest.fixture(name="helper_manager", scope="class")
def helper_manager_fixture():
    """Json Schema test manager fixture"""
    class HelperManager:
        """Json Helper test manager fixture Class"""
        def __init__(self):
            current_script_path = Opath.dirname(
                Opath.abspath(
                    inspect.getfile(inspect.currentframe())
                )
            )
            self.test_path = Opath.join(
                current_script_path,
                "conf"
            )

    return HelperManager()


class TestLoader:
    """Test Loader class."""

    def test_file_not_found(self):
        """Test get_app_blocks_by_app_or_name method """
        file_names = ['nofile.yaml']
        with pytest.raises(YAMLFileNotFound):
            Loader(file_names)

    def test_get_yaml_config(self, helper_manager):
        """Test get_app_blocks_by_app_or_name method """
        file_names = ['dummy_conf_dict.yaml']
        obj = Loader(file_names, file_path=helper_manager.test_path)
        obj.set_file_path(
            file_name=file_names,
            base_path=helper_manager.test_path
        )

        conf = obj.get_yaml_config()
        assert Ut.is_dict(conf) and len(conf) == 7 and \
            len(conf.get('Imports')) == 2 and \
            len(conf.get('Dummy_1')) == 2 and\
            len(conf.get('Dummy_2')) == 2 and\
            len(conf.get('Dummy_3')) == 2 and\
            len(conf.get('Dummy_4')) == 2 and\
            len(conf.get('Dummy_5')) == 2 and\
            len(conf.get('Dummy_6')) == 2

        conf = obj.get_yaml_config(['dummy_conf_dict1.yaml'])
        assert Ut.is_dict(conf) and len(conf) == 5 and \
            len(conf.get('Imports')) == 2 and \
            len(conf.get('Dummy_1')) == 2 and\
            len(conf.get('Dummy_2')) == 2 and\
            len(conf.get('Dummy_3')) == 2 and\
            len(conf.get('Dummy_4')) == 2 and\
            conf.get('Dummy_5') is None and\
            conf.get('Dummy_6') is None

    def test_get_real_file_path(self, helper_manager):
        """Test get_paths_order method """
        file_name = 'dummy_conf_dict.yaml'

        # Test relative path
        test_path = Opath.join('test', 'conf', file_name)
        result = Loader.get_real_file_path(test_path)
        assert Opath.isfile(result)

        # Test absolute path
        abs_path = Opath.join(helper_manager.test_path, file_name)
        result = Loader.get_real_file_path(abs_path)
        assert Opath.isfile(result)

    def test_get_paths_order(self):
        """Test get_paths_order method """
        path_order = Loader.get_paths_order()
        assert Ut.is_list(path_order, not_null=True)

    def test_set_file_path(self, helper_manager):
        """Test set_file_path method """
        file_names = ['dummy_conf_dict.yaml']
        obj = Loader(file_names, file_path=helper_manager.test_path)

        assert Ut.is_str(obj.file_path, not_null=True)

        with pytest.raises(YAMLFileNotFound):
            obj.set_file_path(
                file_name=file_names,
                base_path=''
            )



"""Test SchemaValidate class."""
import inspect
from os import path as Opath
import pytest
from ve_utils.utype import UType as Ut
from jsonschema.exceptions import SchemaError
from vemonitor_m8.conf_manager.shema_validate_selector import SchemaValidate as jValid
from vemonitor_m8.conf_manager.config_loader import ConfigLoader


@pytest.fixture(name="helper_manager", scope="class")
def helper_manager_fixture():
    """Json Schema test manager fixture"""
    class HelperManager:
        """Json Helper test manager fixture Class"""

        def __init__(self):
            current_script_path = Opath.dirname(
                Opath.abspath(
                    inspect.getfile(inspect.currentframe())
                )
            )
            self.test_path = Opath.join(
                current_script_path,
                "conf"
            )

    return HelperManager()


class TestSchemaValidate:
    """Test SchemaValidate class."""

    def test_validate_data(self, helper_manager):
        """Test validate_data method"""
        c_loader = ConfigLoader(helper_manager.test_path)
        settings = c_loader.get_settings_from_schema(
            app_name="batSerialMonitor")
        res = jValid.validate_data(
            data=settings.app_blocks, shem_key_path='appBlocks')
        assert Ut.is_list(res, not_null=True)

        with pytest.raises(SchemaError):
            jValid.validate_data(data=settings.app_blocks,
                                 shem_key_path='bad_schema_key')

    def test_validate_data_from_schema(self, helper_manager):
        """Test validate_data_from_schema method"""
        c_loader = ConfigLoader(helper_manager.test_path)
        settings = c_loader.get_settings_from_schema(
            app_name="batSerialMonitor")
        schema = jValid.load_schema('appBlocks')
        res = jValid.validate_data_from_schema(
            data=settings.app_blocks, schema=schema)
        assert Ut.is_list(res, not_null=True)

        with pytest.raises(SchemaError):
            jValid.validate_data_from_schema(
                data=settings.app_blocks, schema={})

    def test_load_schema(self):
        """Test load_schema method"""
        schema = jValid.load_schema('appBlocks')
        assert Ut.is_dict(schema, not_null=True)

        with pytest.raises(SchemaError):
            jValid.load_schema('')



# -*- coding: utf-8 -*-
"""DictOfObject model helper"""
from abc import ABC, abstractmethod
from typing import Optional
from ve_utils.utype import UType as Ut

__author__ = "Eli Serra"
__copyright__ = "Copyright 2022, Eli Serra"
__deprecated__ = False
__license__ = "Apache"
__status__ = "Production"
__version__ = "1.0.0"


class DictOfObject(ABC):
    """DictOfObject model helper"""
    def __init__(self):
        self.items = None

    def has_items(self) -> bool:
        """Test if items defined."""
        return Ut.is_dict(self.items, not_null=True)

    def has_item_key(self, key: str) -> bool:
        """Test if instance has item key defined."""
        return self.has_items()\
            and self.is_valid_item_key(key)\
            and self.is_valid_item_value(self.items.get(key))

    def init_item(self, reset: bool = False):
        """Initialise item items."""
        if not self.has_items()\
                or reset is True:
            self.items = {}

    def add_item(self,
                 key: any,
                 value: any
                 ) -> bool:
        """Add item item."""
        result = False
        if self.is_valid_item_key(key)\
                and self.is_valid_item_value(value):
            self.init_item()
            self.items[key] = value
            result = True
        return result

    def get_item(self, key: any) -> Optional[object]:
        """Get item key."""
        result = None
        if self.has_item_key(key):
            result = self.items.get(key)
        return result

    def loop_on_items(self):
        """Get item key."""
        if self.has_items():
            for key, item in self.items.items():
                if self.is_valid_item_value(item):
                    yield key, item

    def get_items(self) -> Optional[dict]:
        """Get item key."""
        result = None
        if self.has_items():
            result = self.items
        return result

    @staticmethod
    @abstractmethod
    def is_valid_item_key(key: any):
        """Test if is valid item key"""

    @staticmethod
    @abstractmethod
    def is_valid_item_value(value: any):
        """Test if is valid item value"""



"""Test SchemaValidateSelector class."""
import inspect
from os import path as Opath
import pytest
from jsonschema.exceptions import ValidationError
from vemonitor_m8.conf_manager.shema_validate_selector import SchemaValidateSelector as jValid
from vemonitor_m8.conf_manager.config_loader import ConfigLoader


@pytest.fixture(name="helper_manager", scope="class")
def helper_manager_fixture():
    """Json Schema test manager fixture"""
    class HelperManager:
        """Json Helper test manager fixture Class"""

        def __init__(self):
            current_script_path = Opath.dirname(
                Opath.abspath(
                    inspect.getfile(inspect.currentframe())
                )
            )
            self.test_path = Opath.join(
                current_script_path,
                "conf"
            )

    return HelperManager()


class TestSchemaValidateSelector:
    """Test SchemaValidateSelector class."""

    def test_is_valid_app_blocks_conf(self, helper_manager):
        """Test is_valid_app_blocks_conf method"""
        c_loader = ConfigLoader(helper_manager.test_path)
        settings = c_loader.get_settings_from_schema(
            app_name="batSerialMonitor")
        res = jValid.is_valid_app_blocks_conf(conf_item=settings.app_blocks)
        assert res is True

        with pytest.raises(ValidationError):
            jValid.is_valid_app_blocks_conf(conf_item={})

        with pytest.raises(ValidationError):
            jValid.is_valid_app_blocks_conf(conf_item=None)

    def test_is_valid_app_connectors_conf(self, helper_manager):
        """Test is_valid_app_connectors_conf method"""
        c_loader = ConfigLoader(helper_manager.test_path)
        settings = c_loader.get_settings_from_schema(
            app_name="batSerialMonitor")
        res = jValid.is_valid_app_connectors_conf(
            conf_item=settings.app_connectors)
        assert res is True

        with pytest.raises(ValidationError):
            jValid.is_valid_app_connectors_conf(conf_item={})

        with pytest.raises(ValidationError):
            jValid.is_valid_app_connectors_conf(conf_item=None)

    def test_is_valid_battery_banks_conf(self, helper_manager):
        """Test is_valid_battery_banks_conf method"""
        c_loader = ConfigLoader(helper_manager.test_path)
        imp_conf = c_loader._get_settings_from_file()
        res = jValid.is_valid_battery_banks_conf(
            conf_item=imp_conf.get('batteryBanks'))
        assert res is True

        res = jValid.is_valid_battery_banks_conf(conf_item={})
        assert res is True

        with pytest.raises(ValidationError):
            jValid.is_valid_battery_banks_conf(conf_item={'a': 1})

        with pytest.raises(ValidationError):
            jValid.is_valid_battery_banks_conf(conf_item=None)

    def test_is_valid_data_structure_conf(self, helper_manager):
        """Test is_valid_app_connectors_conf method"""
        c_loader = ConfigLoader(helper_manager.test_path)
        settings = c_loader.get_settings_from_schema(
            app_name="batSerialMonitor")
        res = jValid.is_valid_data_structure_conf(
            conf_item=settings.data_structures)
        assert res is True

        res = jValid.is_valid_data_structure_conf(conf_item={})
        assert res is True

        with pytest.raises(ValidationError):
            jValid.is_valid_data_structure_conf(conf_item={'a': 1})

        with pytest.raises(ValidationError):
            jValid.is_valid_data_structure_conf(conf_item=None)
